%% PARALLELMRFA_P : MRFA  Parallel Analysis (Permutation technique)
% An extension of Horn's method (1965). It uses permutation technique to obtain random samples.
%
% ARGUMENTS   : X        -> Raw sample scores
%               Ndatsets -> Number of random datasets used to compute the random distribution of eigenvalues
%               percent  -> Desired percentile of distribution of random eigenvalues (for example 95 for the 95th percentile) to be used as threshold
%               type     -> Switch between dispersion matrix   
%                               1: Computes Pearson correlation matrices
%                               2: Computes Polychoric correlation matrices
%               display  -> Switch if the results will be displayed   
%                               0: Turns off the screen display of results
%                               1: Turns on the display
%                               2: Turns on the Printing function in an Output listbox
%               Output   -> Optional input, if it's present, the function will print in
%                         that argument. If isn't, the function will display the results in matlab
%                         window, using disp
%               Output2  -> Another optional input, if it's present, the function will be 
%                         printed in LaTeX code in that argument.
%               Output3  -> Another optional input, if it's present, the function will be
%                         printed in rtf code in that argument.
%
% RETURN VALUE: r       -> number of dimensions advised
%               dim     -> number of dimensions
%               realPA  -> vector of real eigenvalues
%               meansPA -> vector of mean random eigenvalues
%
% DEPENDENCIES: ed.m
%               mrfan.m 
%               sr.m
%               polychomap.m
%               ... and 19 more functions, for viewing all type:
%                   matlab.codetools.requiredFilesAndProducts('parallelMRFA_P.m')
%
% Last update: Urbano Lorenzo-Seva 21/04/2015
%

function [r,dim,realPA,meansPA] = parallelMRFA_P(X,Ndatsets, percent, type, display,R_poly,Output,Output2,Output3)

tic;

if display == 1 || display == 2,
   tstart = tic;
   ch=0;
   total=Ndatsets+1;
   tmp3 = 0; bar1 = waitbar(tmp3,'Parallel Analysis','CreateCancelBtn','setappdata(gcbf,''canceling'',1)');
   setappdata(bar1,'canceling',0)
end;    

[N,m]=size(X);
cases=N;

R=corrcoef(X);
if (type==2),
    if isempty(R_poly)

        R=polychomap(X,min(X),max(X),3,0,0);
        if (display >0),
            ch=ch+1;
            compT = toc(tstart);
            compT = compT*(total-ch)/ch;
            s_tmp=['Parallel Analysis (Remaining time = ' Time2String( compT ) ')'];
            waitbar(ch/total,bar1,s_tmp);
       end;
    else
        R=R_poly;
    end;
    
    D=eig(R);
    if min(D) <0,
       %NONLINEAR TRANSFORMATION
       R = sr(R,N);
    end;
end;

[m,tmp] = size(R);  % number of variables.

D=eig(R);
if min(D) <0,
   %NONLINEAR TRANSFORMATION
   R = sr(R,N);
end;

[S, SIGMARED] = mrfan(R, m-1, 10);
[V,L] = ed(SIGMARED);
realevals = diag(L);
si = sum(realevals);
realevals_p = realevals/si*100;

evals=[];
evals_p=[];
for a = 1:Ndatsets,
    
    if (display  >0),
        if getappdata(bar1,'canceling')
            breaking=1;
            break
        end;
    end;
        
    [dummy J]=sort(rand(N,m));	%J is a permutation matrix
    Xi=zeros(N,m);
    
    for ii=1:m, 
        Xi(:,ii)=X(J(:,ii),ii);
    end;
    if (type==2),
        
        Ri=polychomap(Xi,min(Xi),max(Xi),1,0,0,1);
        D=eig(Ri);
        if min(D) <0,
           %NONLINEAR TRANSFORMATION
           Ri = sr(Ri,N);
        end;
    else,
       Ri = corrcoef(Xi);
    end;
    
    D=eig(Ri);
    if min(D) <0,
       %NONLINEAR TRANSFORMATION
       Ri = sr(Ri,N);
    end;
    [S, SIGMARED] = mrfan(Ri, m-1, 10,1); %indiquem que es PA   
    [V,L] = ed(SIGMARED);
    L=diag(L);

    Lp = L/sum(L)*100;

    evals=[evals   L];
    evals_p=[evals_p Lp];
    
    if (display >0),
        ch=ch+1;
        compT = toc(tstart);
        compT = compT*(total-ch)/ch;
        s_tmp=['Parallel Analysis (Remaining time = ' Time2String( compT ) ')'];
        waitbar(ch/total, bar1,s_tmp);
    end;
    
    if a==Ndatsets
        breaking=0;
    end;

end;

if breaking==1
    delete(bar1);
    r=[];
    dim=[];
    realPA=[];
    meansPA=[];
else
    means = (mean(evals,2));   % mean eigenvalues for each position.
    evals = sort(evals,2);     % sorting the eigenvalues for each position.
    percentiles = (evals(:,round((percent*Ndatsets)/100)));  % percentiles.
    means_p = (mean(evals_p,2));   % mean eigenvalues for each position.
    evals_p = sort(evals_p,2);     % sorting the eigenvalues for each position.
    percentiles_p = (evals_p(:,round((percent*Ndatsets)/100)));  % percentiles.

    clear evals
    if display == 1 || display == 2
        delete(bar1);
    end;

    if display==1,
        disp('--------------------------------------------------------------------------------');
            disp(' ');
            disp('PARALLEL ANALYSIS (PA) BASED ON MINIMUM RANK FACTOR ANALYSIS'); 
            disp('(Timmerman & Lorenzo-Seva, 2011)'); disp(' ');
            disp('Implementation details:'); disp(' ');
            if (type==2),
               disp('       Correlation matrices analized:                Polychoric correlation matrices'); 
            else,
               disp('       Correlation matrices analized:                Pearson correlation matrices'); 
            end;
            disp(['       Number of random correlation matrices:        '  num2str(Ndatsets)]);
            disp('       Method to obtain random correlation matrices: Permutation of the raw data (Buja & Eyuboglu, 1992)'); 
            disp(' ');disp(' ');
            disp(['Factor    Real-data        Mean of random   ' num2str(percent) ' percentile of random']);
            disp( '          % of variance    % of variance    % of variance');
            disp(' '); 
            nf_per=0; nf_mean=0;campana=0;
            for i=1:m,
                buff=sprintf(' %3.0f      ',i);
                if campana == 0,
                    if realevals_p(i) > percentiles_p(i),
                       buff2=sprintf('%5.2f**          %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       nf_per=nf_per+1; nf_mean=nf_mean+1;
                    elseif realevals_p(i) > means_p(i),
                       buff2=sprintf('%5.2f*           %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       nf_mean=nf_mean+1;
                    else,
                       buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       campana=1;
                    end;
                else,
                    buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                end;
                disp([buff buff2]);
            end;
            disp(' ');
            if nf_mean > nf_per,
               buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
               disp(buff);
               buff=sprintf('*   Advised number of factors:    %.0f',nf_mean);
               disp(buff);
            else
               buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
               disp(buff);
            end;
            disp(' ');
            %disp(['Time for this problem = ', num2str(toc) ]); disp([' '])
            disp(' ');

    elseif display==2,
        if nargin==7
            MyPrint(Output,' ');
            MyPrint(Output,'PARALLEL ANALYSIS (PA) '); 
            MyPrint(Output,'(Timmerman & Lorenzo-Seva, 2011)');
            MyPrint(Output,' ');
            MyPrint(Output,'Implementation details:');
            MyPrint(Output,' ');
            if type==2
                MyPrint(Output,'       Correlation matrices analized:                Polychoric correlation matrices');
            else
                MyPrint(Output,'       Correlation matrices analized:                Pearson correlation matrices');
            end;
            a=num2str(Ndatsets);
            buff = sprintf('       Number of random correlation matrices:       %s',a); 
            MyPrint(Output, buff);
            MyPrint(Output,'       Method to obtain random correlation matrices: Permutation of the raw data (Buja & Eyuboglu, 1992)');
            MyPrint(Output,' ');
            MyPrint(Output,' ');
            b=num2str(percent);
            buff = sprintf('Factor    Real-data        Mean of random   %s',b);
            buff2 = sprintf('percentile of random');
            MyPrint(Output,[buff buff2]);
            MyPrint(Output, '          % of variance    % of variance    % of variance');
            MyPrint(Output,' ');
            nf_per=0; nf_mean=0;campana=0;
            for i=1:m,
                buff=sprintf(' %3.0f      ',i);
                if campana == 0,
                    if realevals_p(i) > percentiles_p(i),
                       buff2=sprintf('%5.2f**          %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       nf_per=nf_per+1; nf_mean=nf_mean+1;
                    elseif realevals_p(i) > means_p(i),
                       buff2=sprintf('%5.2f*           %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       nf_mean=nf_mean+1;
                    else,
                       buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       campana=1;
                    end;
                else,
                    buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                end;
                MyPrint(Output,[buff buff2]);
            end;
            MyPrint(Output,' ')
            if nf_mean > nf_per,
               buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
               MyPrint(Output,buff);
               buff=sprintf('*   Advised number of factors:    %.0f',nf_mean);
               MyPrint(Output,buff);
            else
               buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
               MyPrint(Output,buff);
            end;
            MyPrint(Output,' ')
            %disp(['Time for this problem = ', num2str(toc) ]); disp([' '])
            MyPrint(Output,'-----------------------------------------------------------------------');
            MyPrint(Output,' ')

        elseif nargin==8
            MyPrint(Output,' ');
            MyPrint(Output,'PARALLEL ANALYSIS (PA) '); 
            MyPrint(Output,'(Timmerman & Lorenzo-Seva, 2011)');
            MyPrint(Output,' ');
            MyPrint(Output,'Implementation details:');
            MyPrint(Output,' ');
            MyPrint(Output2,'\section{Parallel Analysis(PA)\normalsize(Timmerman \& Lorenzo-Seva, 2011)}');
            MyPrint(Output2,'\begin{table}[!htb]');
            MyPrint(Output2,'\caption{PA Implemention Details}');
            MyPrint(Output2,'\label{taula:PAImplemetion}');
            MyPrint(Output2,'\centering');
            MyPrint(Output2,'\begin{tabular}{l l} \hline');
            if type==2
                MyPrint(Output,'       Correlation matrices analized:                Polychoric correlation matrices');
                MyPrint(Output2,'Correlation matrices analized: & Polychoric correlation matrices \\');
            else
                MyPrint(Output,'       Correlation matrices analized:                Pearson correlation matrices');
                MyPrint(Output2,'Correlation matrices analized: & Pearson correlation matrices \\');
            end;
            a=num2str(Ndatsets);
            buff = sprintf('       Number of random correlation matrices:       %s',a); 
            MyPrint(Output, buff);

            buff9='\\';
            buff = sprintf('Number of random correlation matrices: & %s',a);
            MyPrint(Output2, [buff buff9]);
            MyPrint(Output,'       Method to obtain random correlation matrices: Permutation of the raw data (Buja & Eyuboglu, 1992)');
            MyPrint(Output2,'       Method to obtain random correlation matrices: & Permutation of the raw data (Buja \& Eyuboglu, 1992) \\');
            MyPrint(Output,' ');
            MyPrint(Output,' ');
            MyPrint(Output2,'\hline');
            MyPrint(Output2,'\end{tabular}');
            MyPrint(Output2,'\end{table}');
            b=num2str(percent);
            buff = sprintf('Factor    Real-data        Mean of random   %s',b);
            buff2 = sprintf('percentile of random');
            MyPrint(Output,[buff buff2]);
            MyPrint(Output, '          % of variance    % of variance    % of variance');
            MyPrint(Output,' ');
            MyPrint(Output2,'\begin{table}[!htb]');
            MyPrint(Output2,'\caption{Parallel Analyis Output}');
            MyPrint(Output2,'\label{taula:PA}');
            MyPrint(Output2,'\centering');
            MyPrint(Output2,'\begin{tabular}{l c c c}');
            perc='\%';
            MyPrint(Output2,'\hline');
            buff = sprintf('Factor & Real-data %s of variance & Mean of random %s of variance & %s percentile of random %s of variance',perc,perc,b,perc);
            MyPrint(Output2,[buff buff9]);
            MyPrint(Output2,'\hline');
            nf_per=0; nf_mean=0;campana=0;
            for i=1:m,
                buff=sprintf(' %3.0f      ',i);
                if campana == 0,
                    if realevals_p(i) > percentiles_p(i),
                       buff2=sprintf('%5.2f**          %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       lat2=sprintf('& %5.2f** & %5.2f  & %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       nf_per=nf_per+1; nf_mean=nf_mean+1;
                    elseif realevals_p(i) > means_p(i),
                       buff2=sprintf('%5.2f*           %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       lat2=sprintf('& %5.2f*& %5.2f & %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       nf_mean=nf_mean+1;
                    else,
                       buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       lat2=sprintf('& %5.2f & %5.2f & %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       campana=1;
                    end;
                else,
                    buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                    lat2=sprintf('& %5.2f & %5.2f & %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                end;
                MyPrint(Output,[buff buff2]);
                lat3 = '\\';
                MyPrint(Output2,[buff lat2 lat3]);

            end;
            MyPrint(Output2,'\hline');
            MyPrint(Output2,'\end{tabular}');
            MyPrint(Output,' ')
            if nf_mean > nf_per,
               buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
               MyPrint(Output,buff);
               MyPrint(Output2,buff);
               buff=sprintf('*   Advised number of factors:    %.0f',nf_mean);
               MyPrint(Output,buff);
               MyPrint(Output2,buff);
            else
               buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
               MyPrint(Output,buff);
               MyPrint(Output2,buff);
            end;
            MyPrint(Output,' ')
            %disp(['Time for this problem = ', num2str(toc) ]); disp([' '])
            MyPrint(Output,'-----------------------------------------------------------------------');
            MyPrint(Output,' ')
            MyPrint(Output2,'\end{table}');

    %%%%%%%%%%% 3 OUTPUTS %%%%%%%%%%%%

        elseif nargin==9   
            MyPrint(Output,' ');
            MyPrint(Output,'PARALLEL ANALYSIS (PA) '); 
            MyPrint(Output,'(Timmerman & Lorenzo-Seva, 2011)');
            MyPrint(Output,' ');
            MyPrint(Output,'Implementation details:');
            MyPrint(Output,' ');
            MyPrint(Output2,'\section{Parallel Analysis(PA)\normalsize(Timmerman \& Lorenzo-Seva, 2011)}');
            MyPrint(Output2,'\begin{table}[!htb]');
            MyPrint(Output2,'\caption{PA Implemention Details}');
            MyPrint(Output2,'\label{taula:PAImplemetion}');
            MyPrint(Output2,'\centering');
            MyPrint(Output2,'\begin{tabular}{l l} \toprule');

            MyPrint(Output3,'\par');
            MyPrint(Output3,'\qc \ul Parallel Analysis (PA) (Timmerman & Lorenzo-Seva, 2011)\ul0');
            MyPrint(Output3,'\par');
            MyPrint(Output3,'\par');
            interline1='\trowd\trgaph144\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx10000';
            interline2='\trowd\trgaph144\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx5000\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx10000';
            MyPrint(Output3,interline1);

            int='\intbl ';
            cell='\cell';
            row='\row';
            buff='Implementation details \cell \row';
            MyPrint(Output3,[int buff]);
            MyPrint(Output3,'\ql');

            MyPrint(Output3,interline2);
            if type==2
                MyPrint(Output,'       Correlation matrices analized               : Polychoric correlation matrices');
                MyPrint(Output2,'Correlation matrices analized: & Polychoric correlation matrices \\');
                MyPrint(Output3,'\intbl Correlation matrices analized: \cell Polychoric correlation matrices \cell \row');
            else
                MyPrint(Output,'       Correlation matrices analized               : Pearson correlation matrices');
                MyPrint(Output2,'Correlation matrices analized: & Pearson correlation matrices \\');
                MyPrint(Output3,'\intbl Correlation matrices analized: \cell Pearson correlation matrices \cell \row');
            end;
            a=num2str(Ndatsets);
            buff = sprintf('       Number of random correlation matrices       : %s',a); 
            MyPrint(Output, buff);
            buff9='\\';
            buff = sprintf('Number of random correlation matrices: & %s',a);
            MyPrint(Output2, [buff buff9]);

            MyPrint(Output3,interline2);
            buff_rtf=sprintf('Number of random correlation matrices: %s %s %s %s',cell,a,cell,row);
            MyPrint(Output3,[int,buff_rtf]);
            MyPrint(Output,'       Method to obtain random correlation matrices: Permutation of the raw data (Buja & Eyuboglu, 1992)');
            MyPrint(Output2,'       Method to obtain random correlation matrices: & Permutation of the raw data (Buja \& Eyuboglu, 1992) \\');

            MyPrint(Output3,interline2);
            buff_rtf='\intbl Method to obtain random correlation matrices: \cell Permutation of the raw data (Buja & Eyuboglu, 1992) \cell \row \pard \par';
            MyPrint(Output3,buff_rtf);
            MyPrint(Output3,'\par');

            MyPrint(Output,' ');
            MyPrint(Output,' ');
            MyPrint(Output2,'\bottomrule');
            MyPrint(Output2,'\end{tabular}');
            MyPrint(Output2,'\end{table}');
            b=num2str(percent);
            buff = sprintf('Factor    Real-data        Mean of random   %s',b);
            buff2 = sprintf('percentile of random');
            MyPrint(Output,[buff buff2]);
            MyPrint(Output, '          % of variance    % of variance    % of variance');
            MyPrint(Output,' ');
            MyPrint(Output2,'\begin{center}');
            MyPrint(Output2,'\begin{longtable}[H]{l S S S}');
            MyPrint(Output2,'\caption{Parallel Analyis Output} \\');
            MyPrint(Output2,'\toprule');
            perc='\%';
            perc_rtf='%';
            buff = '\multicolumn{1}{l}{\textbf{Factor}} & ';MyPrint(Output2,buff);
            buff = '\multicolumn{1}{p{3.5cm}}{\textbf';
            buff1 = sprintf('{Real-data %s of variance}} & ',perc);MyPrint(Output2,[buff buff1]);
            buff1 = sprintf('{Mean of random %s of variance}} & ',perc);MyPrint(Output2,[buff buff1]);
            buff1 = sprintf('{%s percentile of random %s of variance}} ',b,perc);MyPrint(Output2,[buff buff1 buff9]);
            MyPrint(Output2,'\midrule');
            MyPrint(Output2,'\endfirsthead');
            MyPrint(Output2,'\multicolumn{4}{c}%');
            MyPrint(Output2,'{\tablename\ \thetable\ -- \textit{(Continued from previous page)}} \\');
            MyPrint(Output2,'\toprule');
            buff = '\multicolumn{1}{l}{\textbf{Factor}} & ';MyPrint(Output2,buff);
            buff = '\multicolumn{1}{p{3.5cm}}{\textbf';
            buff1 = sprintf('{Real-data %s of variance}} & ',perc);MyPrint(Output2,[buff buff1]);
            buff1 = sprintf('{Mean of random %s of variance}} & ',perc);MyPrint(Output2,[buff buff1]);
            buff1 = sprintf('{%s percentile of random %s of variance}} ',b,perc);MyPrint(Output2,[buff buff1 buff9]);
            MyPrint(Output2,'\midrule');
            MyPrint(Output2,'\endhead');
            MyPrint(Output2,'\bottomrule \multicolumn{4}{r}{\textit{Continued on next page}} \\');
            MyPrint(Output2,'\endfoot');
            MyPrint(Output2,'\bottomrule');
            MyPrint(Output2,'\endlastfoot');
            
            nf_per=0; nf_mean=0;campana=0;
            for i=1:m,
                buff=sprintf(' %3.0f      ',i);
                if campana == 0,
                    if realevals_p(i) > percentiles_p(i),
                       buff2=sprintf('%5.2f**          %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       lat2=sprintf('& %5.2f** & %5.2f  & %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       nf_per=nf_per+1; nf_mean=nf_mean+1;

                    elseif realevals_p(i) > means_p(i),
                       buff2=sprintf('%5.2f*           %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       lat2=sprintf('& %5.2f*& %5.2f & %5.2f',realevals_p(i), means_p(i), percentiles_p(i));

                       nf_mean=nf_mean+1;
                    else,
                       buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                       lat2=sprintf('& %5.2f & %5.2f & %5.2f',realevals_p(i), means_p(i), percentiles_p(i));

                       campana=1;
                    end;
                else,
                    buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals_p(i), means_p(i), percentiles_p(i));
                    lat2=sprintf('& %5.2f & %5.2f & %5.2f',realevals_p(i), means_p(i), percentiles_p(i));

                end;
                MyPrint(Output,[buff buff2]);
                lat3 = '\\';
                MyPrint(Output2,[buff lat2 lat3]);

            end;
            
            MyPrint(Output,' ');
            MyPrint(Output2,'\end{longtable}'); 
            MyPrint(Output2,'\end{center}');
            
            if nf_mean > nf_per,
               buff=sprintf('**  Advised number of factors:    %.0f ',nf_per);
               MyPrint(Output,buff);
               MyPrint(Output2,[buff lat3]);

               buff=sprintf('*   Advised number of factors:    %.0f',nf_mean);
               MyPrint(Output,buff);
               MyPrint(Output2,[buff lat3]);

            else
               buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
               MyPrint(Output,buff);
               MyPrint(Output2,[buff lat3]);

            end;
            MyPrint(Output,' ')
            %disp(['Time for this problem = ', num2str(toc) ]); disp([' '])
            MyPrint(Output,'-----------------------------------------------------------------------');

            
            

            %%%% RTF %%%%

            X=[realevals_p means_p percentiles_p];
            perc_rtf='%';
            buff={};
            buff{1} = sprintf('Factor');
            buff{2} = sprintf('Real-data %s of variance',perc_rtf);
            buff{3} = sprintf('Mean of random %s of variance',perc_rtf);
            buff{4} = sprintf('%s percentile of random %s of variance',b,perc_rtf);
            header = buff;

            buffs={};
            for i=1:m
                buff = sprintf('%3.0f',i);
                buffs(i)=cellstr(buff);
            end;
        
            Print_RTF(X,Output3,header,buffs); %fixed, no labels
            
            if nf_mean > nf_per,
               buff=sprintf('Advised number of factors: between %.0f and  %.0f',nf_per,nf_mean);
               MyPrint(Output3,buff);
               MyPrint(Output3,'\par');

            else
               buff=sprintf('Advised number of factors: %.0f',nf_per);
               MyPrint(Output3,buff);
               MyPrint(Output3,'\par');

            end;
            MyPrint(Output3,'\par');
        end;
    end;

    r = nf_mean;
    realPA = realevals_p;
    meansPA = means_p;
    dim = m;
end;


